#include <stdio.h>
#include <zlib.h>

#include "inflate.h"

static int fgetw(FILE *fp)
{
	int l = fgetc(fp);
	return l + (fgetc(fp) << 8);
}

static void fputw(FILE *fp, int x)
{
	fputc(x & 0xff, fp);
	fputc(x >> 8, fp);
}

static void fpoke(FILE *fp, int addr, int data)
{
	fputc(0xa9, fp);
	fputc(data, fp);
	fputc(0x85, fp);
	fputc(addr, fp);
}

int main(int argc, char *argv[])
{
	unsigned char inbuf[48000];
	unsigned char outbuf[48000];
	FILE *fp;
	int startaddr;
	int runaddr;
	int inlen;
	z_stream zs;
	int compraddr;
	if (argc != 3) {
		fprintf(stderr, "Usage: zipxex input_file output_file\n");
		return 1;
	}
	fp = fopen(argv[1], "rb");
	if (fp == NULL) {
		perror(argv[1]);
		return 1;
	}
	if (fgetw(fp) != 0xffff) {
		fprintf(stderr, "%s: not Atari executable file\n", argv[1]);
		return 1;
	}
	startaddr = fgetw(fp);
	inlen = fgetw(fp) + 1 - startaddr;
	fread(inbuf, 1, inlen, fp);
	if (fgetw(fp) != 0x2e0 || fgetw(fp) != 0x2e1) {
		fprintf(stderr, "%s: the file should consist of one block plus run address\n", argv[1]);
		return 1;
	}
	runaddr = fgetw(fp);
	if (fgetc(fp) != EOF) {
		fprintf(stderr, "%s: expected EOF after the run address\n", argv[1]);
	}
	fclose(fp);
	zs.next_in = inbuf;
	zs.avail_in = inlen;
	zs.next_out = outbuf;
	zs.avail_out = sizeof(outbuf);
	zs.zalloc = (alloc_func) 0;
	zs.zfree = (free_func) 0;
	if (deflateInit2(&zs, Z_BEST_COMPRESSION, Z_DEFLATED, -MAX_WBITS, 9, Z_DEFAULT_STRATEGY) != Z_OK
	 || deflate(&zs, Z_FINISH) != Z_STREAM_END
	 || deflateEnd(&zs) != Z_OK) {
		fprintf(stderr, "zipxex: deflate failed\n");
		return 1;
	}
	fp = fopen(argv[2], "wb");
	if (fp == NULL) {
		perror(argv[2]);
		return 1;
	}
	fputw(fp, 0xffff);
	compraddr = 0xb6f0 - zs.total_out;
	fputw(fp, compraddr);
	fputw(fp, 0xb700 + sizeof(inflate_obx) - 6 - 1);
	fwrite(outbuf, 1, zs.total_out, fp);
	fpoke(fp, 0xf0, compraddr & 0xff);
	fpoke(fp, 0xf1, compraddr >> 8);
	fpoke(fp, 0xf2, startaddr & 0xff);
	fpoke(fp, 0xf3, startaddr >> 8);
	fwrite(inflate_obx + 6, 1, sizeof(inflate_obx) - 6, fp);
	fputw(fp, 0x2e0);
	fputw(fp, 0x2e3);
	fputw(fp, runaddr);
	fputw(fp, 0xb6f0);
	fclose(fp);
	return 0;
}
